/*
 * The new sysinstall program.
 *
 * This is probably the last attempt in the `sysinstall' line, the next
 * generation being slated for what's essentially a complete rewrite.
 *
 * $Id: options.c,v 1.44.2.10 1997/07/16 05:23:23 jkh Exp $
 *
 * Copyright (c) 1995
 *	Jordan Hubbard.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer,
 *    verbatim and that no modifications are made prior to this
 *    point in the file.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY JORDAN HUBBARD ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL JORDAN HUBBARD OR HIS PETS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, LIFE OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include "sysinstall.h"
#include <ctype.h>

static char *
varCheck(Option opt)
{
    char *cp = NULL;

    if (opt.aux)
	cp = variable_get((char *)opt.aux);
    if (!cp)
	return "NO";
    return cp;
}

/* Show our little logo */
static char *
resetLogo(char *str)
{
    return "[RESET!]";
}

static char *
mediaCheck(Option opt)
{
    if (mediaDevice) {
	switch(mediaDevice->type) {
	case DEVICE_TYPE_UFS:
	case DEVICE_TYPE_DISK:
	    return "File system";

	case DEVICE_TYPE_FLOPPY:
	    return "Floppy";

	case DEVICE_TYPE_FTP:
	    return "FTP";

	case DEVICE_TYPE_CDROM:
	    return "CDROM";

	case DEVICE_TYPE_TAPE:
	    return "Tape";

	case DEVICE_TYPE_DOS:
	    return "DOS";

	case DEVICE_TYPE_NFS:
	    return "NFS";

	case DEVICE_TYPE_NONE:
	case DEVICE_TYPE_NETWORK:
	case DEVICE_TYPE_ANY:
	default:
	    return "<unknown>";
	}
    }
    return "<not yet set>";
}

#define TAPE_PROMPT	"%%13000 Please enter the tape block size in 512 byte blocks:"
#define RELNAME_PROMPT	"%%13001 Please specify the release you wish to load or\n\"none\" for a generic release install:"
#define BPKG_PROMPT	"%%13002 Please specify the name of the HTML browser package:"
#define BBIN_PROMPT	"%%13003 Please specify a full pathname to the HTML browser binary:"
#define EDITOR_PROMPT	"%%13004 Please specify the name of the text editor you wish to use:"
#define PKG_PROMPT	"%%13005 Please specify a temporary directory with lots of free space:"
#define INSTROOT_PROMPT	"%%13006 Please specify a root directory if installing somewhere other than /"
#define TIMEOUT_PROMPT	"%%13030 Please specify the number of seconds to wait for slow media:"
#define GATED_PKG_PROMPT "%%13033 Please specify the package name for the gated software:"
#define PCNFSD_PKG_PROMPT "%%13034 Please specify the package name for the PCNFSD server:"
#ifdef	DOCJP
#define JPHB_PKG_PROMPT "%%13045 Please specify the package name for the Japanese FreeBSD Handbook:"
#define JPHB_PATH_PROMPT "%%13048 Please specify the full pathname for the Japanese FreeBSD Handbook:"
#endif	/* DOCJP */

static Option Options[] = {
{ "NFS Secure",		"%%13007 NFS server talks only on a secure port",
      OPT_IS_VAR,	NULL,			VAR_NFS_SECURE,		varCheck	},
{ "NFS Slow",		"%%13008 User is using a slow PC or ethernet card",
      OPT_IS_VAR,	NULL,			VAR_SLOW_ETHER,		varCheck	},
{ "Debugging",		"%%13009 Emit extra debugging output on VTY2 (ALT-F2)",
      OPT_IS_VAR,	NULL,			VAR_DEBUG,		varCheck	},
{ "Yes to All",		"%%13010 Assume \"Yes\" answers to all non-critical dialogs",
      OPT_IS_VAR,	NULL,			VAR_NO_CONFIRM,		varCheck	},
{ "FTP username",	"%%13011 Username and password to use instead of anonymous",
      OPT_IS_FUNC,	mediaSetFTPUserPass,	VAR_FTP_USER,		varCheck	},
{ "Editor",		"%%13012 Which text editor to use during installation",
      OPT_IS_VAR,	EDITOR_PROMPT,		VAR_EDITOR,		varCheck	},
{ "Tape Blocksize",	"%%13013 Tape media block size in 512 byte blocks",
      OPT_IS_VAR,	TAPE_PROMPT,		VAR_TAPE_BLOCKSIZE,	varCheck	},
{ "Extract Detail",	"%%13014 How verbosely to display file name information during extractions",
      OPT_IS_FUNC,	mediaSetCPIOVerbosity,	VAR_CPIO_VERBOSITY,	varCheck	},
{ "Release Name",	"%%13015 Which release to attempt to load from installation media",
      OPT_IS_VAR,	RELNAME_PROMPT,		VAR_RELNAME,		varCheck	},
{ "Install Root",	"%%13016 Which directory to unpack distributions or packages relative to",
      OPT_IS_VAR,	INSTROOT_PROMPT,	VAR_INSTALL_ROOT,	varCheck	},
{ "Browser package",	"%%13017 This is the browser package that will be used for viewing HTML docs",
      OPT_IS_VAR,	BPKG_PROMPT,		VAR_BROWSER_PACKAGE,	varCheck	},
{ "Browser Exec",	"%%13018 This is the path to the main binary of the browser package",
      OPT_IS_VAR,	BBIN_PROMPT,		VAR_BROWSER_BINARY,	varCheck	},
{ "Media Type",		"%%13019 The current installation media type.",
      OPT_IS_FUNC,	mediaGetType,		VAR_MEDIA_TYPE,		mediaCheck	},
{ "Media Timeout",	"%%13040 Timeout value in seconds for slow media.",
      OPT_IS_VAR,	TIMEOUT_PROMPT,		VAR_MEDIA_TIMEOUT,	varCheck	},
{ "Package Temp",	"%%13020 The directory where package temporary files should go",
      OPT_IS_VAR,	PKG_PROMPT,		VAR_PKG_TMPDIR,		varCheck	},
{ "Gated package",	"%%13043 The name of the gated package to install if requested",
      OPT_IS_VAR,	GATED_PKG_PROMPT,	VAR_GATED_PKG,		varCheck	},
{ "PCNFSD package",	"%%13044 The name of the PCNFSD package to install if requested",
      OPT_IS_VAR,	PCNFSD_PKG_PROMPT,	VAR_PCNFSD_PKG,		varCheck	},
#ifdef	DOCJP
{ "JPHandbook pkg",	"%%13046 The name of the Japanese Handbook package to install if requested",
      OPT_IS_VAR,	JPHB_PKG_PROMPT,	VAR_JPHB_PKG,		varCheck	},
{ "JPHandbook path",	"%%13047 The full pathname of the Japanese Handbook package",
      OPT_IS_VAR,	JPHB_PATH_PROMPT,	VAR_JPHB_PATH,		varCheck	},
#endif	/* DOCJP */
{ "Use Defaults",	"%%13021 Reset all values to startup defaults",
      OPT_IS_FUNC,	installVarDefaults,	0,			resetLogo	},
{ NULL },
};

#define OPT_START_ROW	4
#define OPT_END_ROW	19
#define OPT_NAME_COL	0
#define OPT_VALUE_COL	16
#define GROUP_OFFSET	40

static char *
value_of(Option opt)
{
    static char ival[40];

    switch (opt.type) {
    case OPT_IS_STRING:
	return (char *)opt.data;

    case OPT_IS_INT:
	sprintf(ival, "%d", (int)opt.data);
	return ival;

    case OPT_IS_FUNC:
    case OPT_IS_VAR:
	if (opt.check)
	    return opt.check(opt);
	else
	    return "<*>";
    }
    return "<unknown>";
}

static int
fire(Option opt)
{
    int status = 0;

    if (opt.type == OPT_IS_FUNC) {
	int (*cp)(char *) = opt.data, rcode;

	rcode = cp(NULL);
	if (rcode & (DITEM_RECREATE | DITEM_RESTORE))
	    status = 1;
    }
    else if (opt.type == OPT_IS_VAR) {
	if (opt.data) {
	    (void)variable_get_value(opt.aux, opt.data);
	    status = 1;
	}
	else if (variable_get(opt.aux))
	    variable_unset(opt.aux);
	else
	    variable_set2(opt.aux, "YES");
    }
    if (opt.check)
	opt.check(opt);
    refresh();
    return status;
}

int
optionsEditor(dialogMenuItem *self)
{
    int i, optcol, optrow, key;
    static int currOpt = 0;

    dialog_clear_norefresh();
    clear();

    while (1) {
	/* Whap up the header */
	attrset(A_REVERSE); mvaddstr(0, 0, i18n("%%13022 Options Editor")); attrset(A_NORMAL);
	for (i = 0; i < 2; i++) {
	    mvaddstr(OPT_START_ROW - 2, OPT_NAME_COL + (i * GROUP_OFFSET), i18n("%%13023 Name"));
	    mvaddstr(OPT_START_ROW - 1, OPT_NAME_COL + (i * GROUP_OFFSET), i18nrep("%%13023 Name", '-'));

	    mvaddstr(OPT_START_ROW - 2, OPT_VALUE_COL + (i * GROUP_OFFSET), i18n("%%13024 Value"));
	    mvaddstr(OPT_START_ROW - 1, OPT_VALUE_COL + (i * GROUP_OFFSET), i18nrep("%%13024 Value", '-'));
	}
	/* And the footer */
	mvprintw(OPT_END_ROW + 1, 0, i18n("%%13025 Use SPACE to select/toggle an option, arrow keys to move,"));
	mvprintw(OPT_END_ROW + 2, 0, i18n("%%13026 ? or F1 for more help.  When you're done, type Q to Quit."));

	optrow = OPT_START_ROW;
	optcol = OPT_NAME_COL;
	for (i = 0; Options[i].name; i++) {
	    char tmp[256];
#define MAXVALLENGTH	(GROUP_OFFSET - OPT_VALUE_COL - 3)
	    /* Names are painted somewhat gratuitously each time, but it's easier this way */
	    mvprintw(optrow, OPT_NAME_COL + optcol, Options[i].name);
	    if (currOpt == i)
		attrset(ATTR_SELECTED);
	    strncpy(tmp, value_of(Options[i]), sizeof(tmp));
	    if (strlen(tmp) > MAXVALLENGTH) {
		tmp[MAXVALLENGTH] = '\0';
		tmp[MAXVALLENGTH - 1] = '.';
		tmp[MAXVALLENGTH - 2] = '.';
	    }
	    mvprintw(optrow++, OPT_VALUE_COL + optcol, tmp);
	    if (currOpt == i)
		attrset(A_NORMAL);
	    if (optrow == OPT_END_ROW) {
		optrow = OPT_START_ROW;
		optcol += GROUP_OFFSET;
	    }
	    clrtoeol();
	}
	attrset(ATTR_TITLE);
	mvaddstr(OPT_END_ROW + 4, 0, Options[currOpt].desc);
	attrset(A_NORMAL);
	clrtoeol();
	move(0, 14);
        refresh();

	/* Start the edit loop */
	key = toupper(getch());
#ifdef I18N
	/* Termcap hack */
      termcap_hack:
#endif
	switch (key) {
	case KEY_F(1):
	case '?':
	    systemDisplayHelp("options");
	    clear();
	    break;

	case '\020':	/* ^P */
	case KEY_UP:
#ifdef I18N
	    /* XXX - hack */
	  key_up:
#endif
	    if (currOpt)
		--currOpt;
	    else
		for (currOpt = 0; Options[currOpt + 1].name; currOpt++);
	    continue;

	case '\016':	/* ^N */
	case KEY_DOWN:
#ifdef I18N
	    /* XXX - hack */
	  key_down:
#endif
	    if (Options[currOpt + 1].name)
		++currOpt;
	    else
		currOpt = 0;
	    continue;

	case KEY_HOME:
	    currOpt = 0;
	    continue;

	case KEY_END:
	    while (Options[currOpt + 1].name)
		++currOpt;
	    continue;

	case ' ':
	    if (fire(Options[currOpt]))
	    	clear();
	    continue;

	case '\033':	/* ESC */
#ifdef I18N
	    /* XXX - Termcap cursor key hack */
	    /* I don't know why this code is needed only at this screen */
	    /* (hosokawa) */
	    if (USE_KON_LANG(language)) {
		key = toupper(getch());
		if (key == '[') {
		    key = toupper(getch());
		    switch (key) {
		      case 'A':
			goto key_up;
		      case 'B':
			goto key_down;
		    }
		}
	    }
	    /* FALLTHROUGH */
#endif	/* I18N */
	case 'Q':
	    clear();
	    dialog_clear();
	    return DITEM_SUCCESS | DITEM_RESTORE;

	default:
	    beep();
	}
    }
    /* NOTREACHED */
    return DITEM_SUCCESS | DITEM_RESTORE;
}

#define I18N_DESC_LEN	78
void
options_i18n(void)
{
    Option	*p;
    char	*q;
    int		i, len;

    for (p = Options; p->name; p++) {
	q = strdup(i18n(p->desc));
	p->desc = malloc(I18N_DESC_LEN + 1);
	strcpy(p->desc, q);
	len = strlen(q);
	for (i = 0; i < I18N_DESC_LEN - len; i++) {
	    strcat(p->desc, " ");
	}
	free(q);
    }
}
