/*
 * Copyright (c) 1994, Paul Richards.
 *
 * All rights reserved.
 *
 * This software may be used, modified, copied, distributed, and sold, in both
 * source and binary form provided that the above copyright and these terms
 * are retained, verbatim, as the first lines of this file.  Under no
 * circumstances is the author responsible for the proper functioning of this
 * software, nor does the author assume any responsibility for damages
 * incurred with its use.
 */

#include "sysinstall.h"
#include <stdarg.h>
#include <fcntl.h>
#include <sys/errno.h>
#include <sys/ioctl.h>
#include <machine/console.h>

#define VTY_STATUS_LINE    24
#define TTY_STATUS_LINE    23

static int   cd_stat = 0;
static int   overrided = 0;
static char  *orig_term = 0;
static char  *orig_termcap = 0;

int
set_termcap(void)
{
    char           *term;
    struct ttysize ts;

    term = getenv("TERM");
    cd_stat = ioctl(STDERR_FILENO, GIO_COLOR, &ColorDisplay);

    if (getpid() != 1) {
	if (getenv("SYSINSTALL_DEBUG"))
	    DebugFD = open("sysinstall.debug", O_WRONLY|O_CREAT|O_TRUNC, 0644);
	else
	    DebugFD = -1;
	if (DebugFD < 0)
	    DebugFD = open("/dev/null", O_RDWR, 0);
    }

    if (!OnVTY || (cd_stat < 0)) {
	if (!term) {
	    if (setenv("TERM", "vt100", 1) < 0)
		return -1;
	    if (setenv("TERMCAP", termcap_vt100, 1) < 0)
		return -1;
	}
	if (DebugFD == -1)
	    DebugFD = dup(1);
    }
    else {
	int i, on;

	if (getpid() == 1) {
	    DebugFD = open("/dev/ttyv1", O_WRONLY);
	    on = 1;
	    i = ioctl(DebugFD, TIOCCONS, (char *)&on);
	    msgDebug("ioctl(%d, TIOCCONS, NULL) = %d (%s)\n", DebugFD, i, !i ? "success" : strerror(errno));
	}
	if (ColorDisplay) {
	    if (!term) {
		if (setenv("TERM", "cons25", 1) < 0)
		    return -1;
		if (setenv("TERMCAP", termcap_cons25, 1) < 0)
		    return -1;
	    }
	}
	else {
	    if (!term) {
		if (setenv("TERM", "cons25-m", 1) < 0)
		    return -1;
		if (setenv("TERMCAP", termcap_cons25_m, 1) < 0)
		    return -1;
	    }
	}
    }
    if (ioctl(0, TIOCGSIZE, &ts) == -1) {
	msgDebug("Unable to get terminal size - errno %d\n", errno);
	ts.ts_lines = 0;
    }
    StatusLine = ts.ts_lines ? ts.ts_lines - 1: (OnVTY ? VTY_STATUS_LINE : TTY_STATUS_LINE);
    return 0;
}

int    termcap_resetted = 0;

/* override TERMCAP entryies for L10N support */
int 
override_termcap(void)
{
    char           *term;

    termcap_resetted = 0;
    if (!orig_term && !orig_termcap) {
	char    *term;
	char    *termcap;

	term = getenv("TERM");
	if (term) {
	    orig_term = strdup(term);
	}
	termcap = getenv("TERMCAP");
	if (termcap) {
	    orig_termcap = strdup(termcap);
	}
    }
    if (cd_stat < 0) {
	return 1;
    }
    term = getenv("TERM");
    msgDebug("Term = %s\n", term);
    if (ColorDisplay) {
	if (!term ||
		strncmp(term, "kons", 4) == 0 || 
		strncmp(term, "cons", 4) == 0) {
	    if (USE_KON_LANG(language)) {
		if (setenv("TERM", "kons25jx", 1) < 0)
		  return -1;
		if (setenv("TERMCAP", termcap_kons25jx, 1) < 0)
		  return -1;
		overrided = 1;
	    }
	}
    }
    else {
	if (!term ||
		strncmp(term, "kons", 4) == 0 || 
		strncmp(term, "cons", 4) == 0) {
	    if (USE_KON_LANG(language)) {
		if (setenv("TERM", "kons25jx-m", 1) < 0)
		  return -1;
		if (setenv("TERMCAP", termcap_kons25jx_m, 1) < 0)
		  return -1;
		overrided = 1;
	    }
	}
    }

    if (overrided) {
	init_dialog();
	DialogActive = 1;
    }
    return 0;
}

int
reset_termcap(void)
{
    termcap_resetted = 1;

    if (orig_term) {
	if (setenv("TERM", orig_term, 1) < 0) {
	    return -1;
	}
    }
    if (orig_termcap) {
	if (setenv("TERMCAP", orig_termcap, 1) < 0) {
	    return -1;
	}
    }
    return 0;
}
